'use strict';
const os = require('node:os');                 // operating system related functionalities from node
const fs = require('node:fs');                 // file system related operations from node
const path = require('node:path');             // file and directory path operations from node
const process = require('node:process');       // current process related functionalities from node
const http = require('node:http');             // node http server functionalities
const https = require('node:https');           // node https server functionalities
const Koa = require('koa');                    // web app HTTP(S) middleware framework
const Pug = require('koa-pug');                // HTML templating for Koa and using the Pug framework
const favicon = require('koa-favicon');        // support for a favicon file in koa
const logger = require('koa-logger');          // simple logger package for koa
const bodyParser = require('koa-bodyparser');  // parses payloads on requests and builds the ctx.body object from JSON or url encoded fields
const router = require('./middleware/router'); // the router definition (using koa-router)
const glob = require('./global');

const webSocket = require('ws');

const config = {
  domain: os.hostname().toLowerCase(),
  http: {
    port: 3001
  },
  /*
  https: {
    port: 3000,
    options: {
      key: fs.readFileSync(path.resolve(process.cwd(), 'certs/server.key'), 'utf8').toString(),
      cert: fs.readFileSync(path.resolve(process.cwd(), 'certs/server.crt'), 'utf8').toString()
    }
  }
  */
};

const app = new Koa();
const koaCallback = app.callback();
const pug = new Pug({
  viewPath: path.resolve(process.cwd(), 'views'),
  basedir: path.resolve(process.cwd(), 'views'),
  app: app
});

app.use(logger());                                          // simple logger writing on the console
app.use(favicon(__dirname + '/favicon.ico'));               // serve the favicon on every page
app.use(bodyParser());                                      // builds the ctx.body object with the payload (JSON or URL encoded properties)
app.use(router.routes());                                   // responses to HTTP(S) requests
app.use(router.allowedMethods());

const wss = new webSocket.Server({port: 8000});

wss.on('connection', function(soc) {
  glob.socket = soc;
  console.log('Client has connected');
  soc.on('message', function(msg) {
    console.log('Received: ', msg.toString());
  });
});

try {
  const httpServer = http.createServer(koaCallback);
  httpServer.listen(config.http.port, function(err) {
    if (err)
      console.error('HTTP fail: ', err, err.stack);
    else
      console.log('Server running on http://' + config.domain + ':' + config.http.port);
  });
}
catch (excp) {
  console.error('Exited: ', excp, (excp && excp.statck));
}

/*
try {
  const httpsServer = https.createServer(config.https.options, koaCallback);
  httpsServer.listen(config.https.port, function(err) {
    if (err)
      console.error('HTTPS fail: ', err, err.stack);
    else
      console.log('Server running on https://', config.domain + ':' + config.https.port);
  });
}
catch (excp) {
  console.error('Exited: ', excp, (excp && excp.stack));
}
*/